-- Use three characters for the currency code (ISO 4217)
ALTER TABLE business ALTER COLUMN defaultcurrency VARCHAR(3);
ALTER TABLE currency ALTER COLUMN code VARCHAR(3);
ALTER TABLE catalogitem ALTER COLUMN currency VARCHAR(3);
ALTER TABLE client ALTER COLUMN taskpricecurrency VARCHAR(3);
ALTER TABLE deposit ALTER COLUMN currency VARCHAR(3);
ALTER TABLE expense ALTER COLUMN currency VARCHAR(3);
ALTER TABLE expense ALTER COLUMN totalcurrency VARCHAR(3);
ALTER TABLE taxamount ALTER COLUMN currency VARCHAR(3);
ALTER TABLE invoice ALTER COLUMN currency VARCHAR(3);
ALTER TABLE invoice ALTER COLUMN totaldiscountcurrency VARCHAR(3);
ALTER TABLE item ALTER COLUMN currency VARCHAR(3);
ALTER TABLE item ALTER COLUMN ratediscountcurrency VARCHAR(3);
ALTER TABLE exchangerate ALTER COLUMN source VARCHAR(3);
ALTER TABLE exchangerate ALTER COLUMN target VARCHAR(3);
ALTER TABLE payment ALTER COLUMN currency VARCHAR(3);
ALTER TABLE product ALTER COLUMN currency VARCHAR(3);
ALTER TABLE product ALTER COLUMN pricediscountcurrency VARCHAR(3);
ALTER TABLE project ALTER COLUMN taskpricecurrency VARCHAR(3);
ALTER TABLE recurringinvoice ALTER COLUMN currency VARCHAR(3);
ALTER TABLE recurringinvoiceitem ALTER COLUMN pricediscountcurrency VARCHAR(3);
ALTER TABLE recurringinvoiceitem ALTER COLUMN pricecurrency VARCHAR(3);
ALTER TABLE task ALTER COLUMN pricediscountcurrency VARCHAR(3);
ALTER TABLE task ALTER COLUMN pricecurrency VARCHAR(3);
ALTER TABLE taskcategory ALTER COLUMN pricecurrency VARCHAR(3);
ALTER TABLE triprate ALTER COLUMN currency VARCHAR(3);

-- Use two characters for distance units (km, mi)
ALTER TABLE business ALTER COLUMN defaultdistanceunit VARCHAR(2);
ALTER TABLE trip ALTER COLUMN distanceunit VARCHAR(2);
ALTER TABLE triprate ALTER COLUMN distanceunit VARCHAR(2);

-- Use no more than 35 characters for locale (BCP47/RFC5646)
ALTER TABLE client ALTER COLUMN locale VARCHAR(35);

-- Use 255 characters for email, other
ALTER TABLE business ALTER COLUMN email VARCHAR(255);
ALTER TABLE business ALTER COLUMN other VARCHAR(255);
ALTER TABLE client ALTER COLUMN email VARCHAR(255);
ALTER TABLE client ALTER COLUMN other VARCHAR(255);

-- Use 255 characters for website (RFC1035)
ALTER TABLE business ALTER COLUMN website VARCHAR(255);
ALTER TABLE client ALTER COLUMN website VARCHAR(255);

-- Use more characters for address-related columns
ALTER TABLE business ALTER COLUMN address VARCHAR(255);
ALTER TABLE business ALTER COLUMN city VARCHAR(100);
ALTER TABLE business ALTER COLUMN country VARCHAR(100);
ALTER TABLE business ALTER COLUMN state VARCHAR(100);
ALTER TABLE client ALTER COLUMN address VARCHAR(255);
ALTER TABLE client ALTER COLUMN city VARCHAR(100);
ALTER TABLE client ALTER COLUMN country VARCHAR(100);
ALTER TABLE client ALTER COLUMN state VARCHAR(100);
ALTER TABLE project ALTER COLUMN location VARCHAR(255);

-- Use more characters for internal identifiers
ALTER TABLE timeentry ALTER COLUMN externalid VARCHAR(255);
ALTER TABLE client ALTER COLUMN externalid VARCHAR(255);

-- Use more characters for catalog item codes (sku)
ALTER TABLE catalogitem ALTER COLUMN code VARCHAR(100);

-- Use 100 characters for identifiers (numbers and references)
ALTER TABLE deposit ALTER COLUMN reference VARCHAR(100);
ALTER TABLE expense ALTER COLUMN reference VARCHAR(100);
ALTER TABLE invoice ALTER COLUMN number VARCHAR(100);
ALTER TABLE invoice ALTER COLUMN reference VARCHAR(100);
ALTER TABLE payment ALTER COLUMN reference VARCHAR(100);
ALTER TABLE project ALTER COLUMN number VARCHAR(100);
ALTER TABLE project ALTER COLUMN reference VARCHAR(100);
ALTER TABLE task ALTER COLUMN reference VARCHAR(100);

-- Use more characters for invoice period
ALTER TABLE invoice ALTER COLUMN period VARCHAR(255);

-- Use 255 characters for names (people, companies, products, services)
ALTER TABLE business ALTER COLUMN attention VARCHAR(255);
ALTER TABLE business ALTER COLUMN name VARCHAR(255);
ALTER TABLE catalogitem ALTER COLUMN name VARCHAR(255);
ALTER TABLE client ALTER COLUMN attention VARCHAR(255);
ALTER TABLE client ALTER COLUMN legalname VARCHAR(255);
ALTER TABLE client ALTER COLUMN name VARCHAR(255);
ALTER TABLE expensecategory ALTER COLUMN name VARCHAR(255);
ALTER TABLE invoice ALTER COLUMN attention VARCHAR(255);
ALTER TABLE item ALTER COLUMN name VARCHAR(255);
ALTER TABLE product ALTER COLUMN name VARCHAR(255);
ALTER TABLE project ALTER COLUMN name VARCHAR(255);
ALTER TABLE recurringinvoice ALTER COLUMN name VARCHAR(255);
ALTER TABLE recurringinvoiceitem ALTER COLUMN name VARCHAR(255);
ALTER TABLE task ALTER COLUMN name VARCHAR(255);
ALTER TABLE taskcategory ALTER COLUMN name VARCHAR(255);
ALTER TABLE tax ALTER COLUMN name VARCHAR(255);
ALTER TABLE taxgroup ALTER COLUMN name VARCHAR(255);
ALTER TABLE triprate ALTER COLUMN name VARCHAR(255);

-- Use 4000 characters for descriptions and notes
ALTER TABLE catalogitem ALTER COLUMN description VARCHAR(4000);
ALTER TABLE client ALTER COLUMN notes VARCHAR(4000);
ALTER TABLE deposit ALTER COLUMN notes VARCHAR(4000);
ALTER TABLE expense ALTER COLUMN description VARCHAR(4000);
ALTER TABLE expense ALTER COLUMN notes VARCHAR(4000);
ALTER TABLE invoice ALTER COLUMN notes VARCHAR(4000);
ALTER TABLE item ALTER COLUMN description VARCHAR(4000);
ALTER TABLE payment ALTER COLUMN notes VARCHAR(4000);
ALTER TABLE product ALTER COLUMN description VARCHAR(4000);
ALTER TABLE project ALTER COLUMN description VARCHAR(4000);
ALTER TABLE project ALTER COLUMN notes VARCHAR(4000);
ALTER TABLE recurringinvoiceitem ALTER COLUMN description VARCHAR(4000);
ALTER TABLE task ALTER COLUMN description VARCHAR(4000);
ALTER TABLE task ALTER COLUMN notes VARCHAR(4000);
ALTER TABLE taskcategory ALTER COLUMN description VARCHAR(4000);
ALTER TABLE timeentry ALTER COLUMN description VARCHAR(4000);
ALTER TABLE trip ALTER COLUMN description VARCHAR(4000);

-- Represent monetary amounts using NUMERIC(21,6)
ALTER TABLE catalogitem ALTER COLUMN rate NUMERIC(21,6);
ALTER TABLE client ALTER COLUMN taskprice NUMERIC(21,6);
ALTER TABLE deposit ALTER COLUMN amount NUMERIC(21,6);
ALTER TABLE exchangerate ALTER COLUMN rate NUMERIC(21,6);
ALTER TABLE expense ALTER COLUMN amount NUMERIC(21,6);
ALTER TABLE expense ALTER COLUMN totalamount NUMERIC(21,6);
ALTER TABLE taxamount ALTER COLUMN amount NUMERIC(21,6);
ALTER TABLE invoice ALTER COLUMN totaldiscountrate NUMERIC(21,6);
ALTER TABLE item ALTER COLUMN rate NUMERIC(21,6);
ALTER TABLE item ALTER COLUMN ratediscountrate NUMERIC(21,6);
ALTER TABLE payment ALTER COLUMN amount NUMERIC(21,6);
ALTER TABLE product ALTER COLUMN amount NUMERIC(21,6);
ALTER TABLE product ALTER COLUMN pricediscountamount NUMERIC(21,6);
ALTER TABLE project ALTER COLUMN taskprice NUMERIC(21,6);
ALTER TABLE recurringinvoiceitem ALTER COLUMN pricediscountrate NUMERIC(21,6);
ALTER TABLE recurringinvoiceitem ALTER COLUMN priceamount NUMERIC(21,6);
ALTER TABLE task ALTER COLUMN pricediscountamount NUMERIC(21,6);
ALTER TABLE task ALTER COLUMN priceamount NUMERIC(21,6);
ALTER TABLE taskcategory ALTER COLUMN priceamount NUMERIC(21,6);
ALTER TABLE triprate ALTER COLUMN rate NUMERIC(21,6);

-- Represent quantities using NUMERIC(10,2)
ALTER TABLE item ALTER COLUMN units NUMERIC(10,2);
ALTER TABLE item ALTER COLUMN unitsdiscountfixedvalue NUMERIC(10,2);
ALTER TABLE product ALTER COLUMN regularquantity NUMERIC(10,2);
ALTER TABLE product ALTER COLUMN quantitydiscountfixedvalue NUMERIC(10,2);
ALTER TABLE task ALTER COLUMN units NUMERIC(10,2);
ALTER TABLE task ALTER COLUMN quantitydiscountfixedvalue NUMERIC(10,2);
ALTER TABLE recurringinvoiceitem ALTER COLUMN units NUMERIC(10,2);
ALTER TABLE recurringinvoiceitem ALTER COLUMN unitsdiscountfixedvalue NUMERIC(10,2);

-- Represent discount percentages using NUMERIC(6,4)
ALTER TABLE invoice ALTER COLUMN totaldiscountpercentagevalue NUMERIC(6,4);
ALTER TABLE item ALTER COLUMN unitsdiscountpercentagevalue NUMERIC(6,4);
ALTER TABLE item ALTER COLUMN ratediscountpercentagevalue NUMERIC(6,4);
ALTER TABLE product ALTER COLUMN quantitydiscountpercentagevalue NUMERIC(6,4);
ALTER TABLE product ALTER COLUMN pricediscountpercentagevalue NUMERIC(6,4);
ALTER TABLE task ALTER COLUMN quantitydiscountpercentagevalue NUMERIC(6,4);
ALTER TABLE task ALTER COLUMN pricediscountpercentagevalue NUMERIC(6,4);
ALTER TABLE recurringinvoiceitem ALTER COLUMN unitsdiscountpercentagevalue NUMERIC(6,4);
ALTER TABLE recurringinvoiceitem ALTER COLUMN pricediscountpercentagevalue NUMERIC(6,4);

-- Represent tax rates using NUMERIC(6,5)
ALTER TABLE taxrate ALTER COLUMN rate NUMERIC(6,5);

-- Represent distance using NUMERIC(10,2)
ALTER TABLE trip ALTER COLUMN distance NUMERIC(10,2);